/*
t-sql scripts for ADFv2 import control
Delora J Bradish
www.delorabradish.com
*/

CREATE TABLE [etl].[ADFControl](
	[RowId] [int] IDENTITY(1,1) NOT NULL,
	[DataSource] [nvarchar](25) NOT NULL,
	[CoreFileName] [nvarchar](100) NOT NULL,
	[ColumnList] [nvarchar](4000) NULL,
	[LastExtractStartDateTime] [datetime] NOT NULL,
	[LastExtractEndDateTime] [datetime] NULL,
	[RowInsertDateTime] [datetime] NOT NULL,
PRIMARY KEY CLUSTERED 
(
	[RowId] ASC
)WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO

ALTER TABLE [elt].[ADFControl] ADD  CONSTRAINT [DF_ADFControl_RowInsertDateTime]  DEFAULT (getutcdate()) FOR [RowInsertDateTime]
GO


CREATE VIEW [etl].[vwADFControl] as
SELECT
	 RowId
	,DataSource
	,[CoreFileName]
	,[ColumnList]
	,LastExtractStartDateTime
	,LastExtractEndDateTime
	,ExecutionDuration = DateDiff(minute, LastExtractEndDateTime, LastExtractStartDateTime)
	,NextExtractStartDateTime =IsNull(LastExtractStartDateTime, '1/2/1900') -.001
	,RowInsertDateTime
FROM etl.ADFControl 
GO

--used for ADFv2 lookup activity
CREATE PROC [etl].[usp_ADF_Get_ADFControl_NextExtract] 
	 @DataSource [NVARCHAR](25)
	,@CoreFileName [NVARCHAR](100) 
AS
SELECT top 1 NextExtractStartDateTime
FROM etl.vwADFControl
WHERE  DataSource = @DataSource
   AND CoreFileName = @CoreFileName
GO


--used for ADFv2 execute storec procedure activity
CREATE PROC [etl].[usp_ADF_Set_ADFControl] 
	 @UpdateActionType [CHAR](1)
	,@DataSource [NVARCHAR](25)
	,@CoreFileName [NVARCHAR](100)
	,@ExtractDateTime [NVARCHAR](50) 

AS

BEGIN

	DECLARE @ExtractDateTime2 DATETIME
	IF ISDATE(@ExtractDateTime) = 1
		SET @ExtractDateTime2 = CAST(@ExtractDateTime as DATETIME)

	--Insert source control row if not found (first ADF execution)
	IF NOT EXISTS (	SELECT top 1 *
					FROM etl.ADFControl 
					WHERE   [DataSource]	= @DataSource	
						AND [CoreFileName]	= @CoreFileName	
					)
		INSERT etl.ADFControl(DataSource, CoreFileName, LastExtractStartDateTime)
		VALUES (@DataSource, @CoreFileName, @ExtractDateTime)

	IF @UpdateActionType = 'S' 
		UPDATE etl.ADFControl
		SET LastExtractStartDateTime = @ExtractDateTime2
		  , LastExtractEndDateTime = NULL
		WHERE   [DataSource]	= @DataSource						
			AND [CoreFileName]	= @CoreFileName	

	IF @UpdateActionType = 'E'
		BEGIN
			UPDATE etl.ADFControl
			SET LastExtractEndDateTime = @ExtractDateTime2
			WHERE   [DataSource]	= @DataSource						
				AND [CoreFileName]	= @CoreFileName	
		
			--Add INSERT to history table if desired
		END						

END
GO


--used for ADFv2 lookup activity
CREATE PROC [etl].[usp_ADF_Get_ADFControl_ColumnList] 
	 @DataSource [NVARCHAR](25)
	,@CoreFileName [NVARCHAR](100) 
AS
SELECT top 1 ColumnList
FROM etl.vwADFControl
WHERE  DataSource = @DataSource
   AND CoreFileName = @CoreFileName
GO